/**
 * 
 */
package demo.model;

import java.util.Date;

import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.EventQueues;
import org.zkoss.zk.ui.ext.Scope;

/**
 * @author Ian YT Tsai(zanyking)
 *
 */
public class UserManager {

	private User user;
	
	public static UserManager getInstance(){
		Scope scope = Executions.getCurrent().getSession();
		UserManager manager = (UserManager) scope.getAttribute("userManager");
		if(manager == null){
			scope.setAttribute("userManager", manager = new UserManager());
		}
		return manager;
	}

	public User getUser(){
		return user;
	}
	
	public boolean hasJoined(){
		return user!=null;
	}
	
	private static final String USER_EVENT_QUEUE = "USER_EVENT_QUEUE";
	
	public static final String ON_USER_JOIN = "onUserJoin";
	public static final String ON_USER_LEAVE = "onUserLeave";
	
	public void join(String name, String email){
		if(user!=null || name==null || email==null){
			return ;
		}
		user = new User();
		user.setEmail(email);
		user.setJoinDate(new Date());
		user.setName(name);
		MessageboardManager.getInstance().getMessageboard().join(user);
		EventQueues.lookup(USER_EVENT_QUEUE).publish(new Event(ON_USER_JOIN, null, user));
	}
	
	public void leave(){
		if(user==null)return;
		User oriUser = user;
		user = null;
		MessageboardManager.getInstance().getMessageboard().leave(oriUser);
		EventQueues.lookup(USER_EVENT_QUEUE).publish(new Event(ON_USER_LEAVE, null, oriUser));
		
	}
	
	public void subscribe(EventListener listener){
		EventQueues.lookup(USER_EVENT_QUEUE).subscribe(listener);
	}
	public void unsubscribe(EventListener listener){
		EventQueues.lookup(USER_EVENT_QUEUE).unsubscribe(listener);
	}
	
}
